@@ -41,7 +41,7 @@ try:
 except ImportError:
 	print 'Psyco not installed, the program will just run slower'
 
-print "DeVeDe 3.7"
+print "DeVeDe 3.8"
 
 # append the directories where we install the devede's own modules
 tipo=-1
@@ -139,7 +139,7 @@ arbol=gtk.glade.XML(glade,domain="devede")
 _ = gettext.gettext
 
 try:
-	from devede_other import *
+	import devede_other
 except:
 	print "Failed to load modules DEVEDE_OTHER. Exiting"
 	sys.exit(1)
@@ -166,11 +166,7 @@ except:
 	sys.exit(1)
 
 
-# TODO Change the following to work on both linux and windows
-# add detection to get_win32_home_directory for win and linux and change it to
-# get_home_directory
-
-home=get_home_directory()
+home=devede_other.get_home_directory()
 
 #locale.setlocale(locale.LC_ALL,"")
 #gettext.textdomain('devede')
@@ -195,7 +191,13 @@ global_vars["menu_PAL"]=True
 global_vars["gladefile"]=glade
 global_vars["erase_temporary_files"]=True
 global_vars["number_actions"]=1
+global_vars["menu_bgcolor"]=[0,0,0,49152]
+global_vars["menu_font_color"]=[65535,65535,65535,65535]
+global_vars["menu_selc_color"]=[0,65535,65535,65535]
+global_vars["menu_alignment"]=2 # middle
 global_vars["expand_advanced"]=False
+global_vars["erase_files"]=True
+global_vars["action_todo"]=2
 
 if font_path[-1]!=os.sep:
 	font_path+=os.sep
@@ -204,23 +206,20 @@ global_vars["font_path"]=font_path
 
 #if our font for subtitles isn't installed, we install it
 
-#TODO
 try:
-	if not sys.platform=='win32':
+	if sys.platform!='win32':
 		fichero=open(home+".spumux/devedesans.ttf")
 		fichero.close()
-	else: # get home		
-		t=r'C:\WINDOWS\Fonts\devedesans.tff'
+	else: # get home
+		t=os.path.join(os.environ["WINDIR"], "Fonts", "devedesans.ttf")
 		fichero=open(t)
 		fichero.close()
 except:
 	if sys.platform=='win32':
-		t=os.path.join("C:\WINDOWS", "Fonts" )#=r'C:\WINDOWS\Fonts\\'
+		t=os.path.join(os.environ["WINDIR"], "Fonts" )#=r'C:\WINDOWS\Fonts\\'
 		# spummux needs font in the windows font directory
 		print "\n\nFont Path: " , font_path
 		print "\n\nT: " , t
-		if not os.path.isdir(t):
-			os.mkdir(t)
 		shutil.copyfile(font_path,t+os.sep+"devedesans.ttf")
 	else:
 		handle=subprocess.Popen("mkdir "+home+".spumux",bufsize=8192,shell=True)
@@ -228,47 +227,53 @@ except:
 		handle=subprocess.Popen("cp "+font_path+" "+home+".spumux/",bufsize=8192,shell=True)
 		handle.wait()
 
-load_config(global_vars) # load configuration
+devede_other.load_config(global_vars) # load configuration
 
 errors="" # check for installed programs
 if not sys.platform=='win32': 
-	if 127==check_program("mplayer -v"):
+	if 127==devede_other.check_program("mplayer -v"):
 		errors+="mplayer\n"
-	if 127==check_program("mencoder -msglevel help"):
+	if 127==devede_other.check_program("mencoder -msglevel help"):
 		errors+="mencoder\n"
-	if 127==check_program("dvdauthor --help"):
+	if 127==devede_other.check_program("dvdauthor --help"):
 		errors+="dvdauthor\n"
-	if 127==check_program("vcdimager --help"):
+	if 127==devede_other.check_program("vcdimager --help"):
 		errors+="vcdimager\n"
-	if 127==check_program("mkisofs -help"):
+	if 127==devede_other.check_program("mkisofs -help"):
 		errors+="mkisofs\n"
-	if 127==check_program("spumux --help"):
+	if 127==devede_other.check_program("spumux --help"):
 		errors+="spumux\n"
+	if 127==devede_other.check_program("convert --help"):
+		errors+="convert (imagemagick)\n"
 else:
 	try:
-		check_program(["mplayer.exe", "-v"])
+		devede_other.check_program(["mplayer.exe", "-v"])
 	except:
 		errors+="mplayer\n"
 	try:
-		check_program(["mencoder.exe", "-msglevel", "help"])
+		devede_other.check_program(["mencoder.exe", "-msglevel", "help"])
 	except:
 		errors+="mencoder\n"
 	try:
-		check_program(["dvdauthor.exe", "--help"])
+		devede_other.check_program(["dvdauthor.exe", "--help"])
 	except:
 		errors+="dvdauthor\n"
 	try:
-		check_program(["vcdimager.exe", "--help"])
+		devede_other.check_program(["vcdimager.exe", "--help"])
 	except:
 		errors+="vcdimager\n"
 	try:
-		check_program(["mkisofs.exe"])
+		devede_other.check_program(["mkisofs.exe"])
 	except:
 		errors+="mkisofs\n"
 	try:
-		check_program(["spumux.exe", "--help"])
+		devede_other.check_program(["spumux.exe", "--help"])
 	except:
 		errors+="spumux\n"
+	try:
+		devede_other.check_program(["convert.exe", "--help"])
+	except:
+		errors+="convert (imagemagick)\n"
 
 def program_exit(widget):
 	
@@ -286,4 +291,4 @@ else:
 	new_file=devede_disctype.disctype(global_vars)
 
 gtk.main()
-save_config(global_vars)
+devede_other.save_config(global_vars)
