@@ -25,6 +25,7 @@ import gtk
 import gtk.glade
 import gobject
 import os
+import sys
 
 import devede_other
 import devede_newfiles
@@ -44,7 +45,11 @@ class main_window:
 		self.tree=gtk.glade.XML(self.gladefile,"wmain",domain="devede")
 		self.tree.signal_autoconnect(self)
 		self.window=self.tree.get_widget("wmain")
-		self.window.drag_dest_set(gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP,[ ( "text/plain", 0, 80 ) ],gtk.gdk.ACTION_COPY)
+		if sys.platform=="win32":
+			self.window.drag_dest_set(0,[],0)
+			self.window.connect('drag_drop', self.drop_cb)
+		else:
+			self.window.drag_dest_set(gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP,[ ( "text/plain", 0, 80), ( "video/*", 0, 81) ],gtk.gdk.ACTION_COPY)
 		self.global_vars["main_window"]=self.window
 		
 		self.list_titles=gtk.TreeStore(gobject.TYPE_PYOBJECT,gobject.TYPE_STRING)
@@ -72,6 +77,35 @@ class main_window:
 		self.callback=callback # callback to call to show again the SELECT_DISK dialog
 		
 		self.window.show()
+		
+		w=self.tree.get_widget("multicore")
+		if (self.global_vars["multicore"]==1):
+			w.set_active(False)
+		else:
+			w.set_active(True)
+			self.global_vars["multicore"]=self.get_cores()
+		
+		print "Cores: "+str(self.get_cores())
+
+
+	def hide(self):
+		if self.window!=None:
+			self.window.hide()
+
+
+	def show(self):
+		if self.window!=None:
+			self.window.show()
+
+
+	def drop_cb(self, wid, context, x, y, time):
+		# Used with windows drag and drop
+		print 'drop'
+		self.have_drag = False
+		if context.targets:
+			wid.drag_get_data(context, context.targets[0], time)
+			return True
+		return False
 
 
 	def set_default_global(self):
@@ -87,6 +121,8 @@ class main_window:
 
 	def set_disc_type(self,delete_structure):
 		
+		""" Changes the interface to adjust to the disk type """
+		
 		if delete_structure:
 			self.structure=[]
 			self.structure.append(self.create_new_structure(1))
@@ -99,7 +135,13 @@ class main_window:
 			w.set_active(4)
 		else:
 			w.set_active(2)
-			
+		
+		w = self.tree.get_widget("autosize")
+		if (self.disctocreate == "vcd"):
+			w.hide()
+		else:
+			w.show()
+		
 		w1 = self.tree.get_widget("frame1") # contains the titles
 		w2 = self.tree.get_widget("create_dvd") # toggle to create the DVD structure
 		w4 = self.tree.get_widget("domenu") # checkbox to create/not create the menu
@@ -126,20 +168,26 @@ class main_window:
 			w6.hide()
 		
 		w1 = self.tree.get_widget("frame5") # ACTION frame
-		w2 = self.tree.get_widget("frame19") # OPTIONS frame
+		w2 = self.tree.get_widget("erase_files")
 		if self.disctocreate == "divx":
 			w1.hide()
 			w2.hide()
+			self.global_vars["action_todo"]=0
 			w = self.tree.get_widget("only_convert")
 			w.set_active(True)
 		else:
 			w1.show()
 			w2.show()
-			w=self.tree.get_widget("create_iso")
+			if self.global_vars["action_todo"]==2:
+				w=self.tree.get_widget("create_iso")
+			elif self.global_vars["action_todo"]==1:
+				w=self.tree.get_widget("create_dvd")
+			else:
+				w=self.tree.get_widget("only_convert")
 			w.set_active(True)
 		
 		w=self.tree.get_widget("erase_files")
-		w.set_active(True)
+		w.set_active(self.global_vars["erase_files"])
 	
 		# now select the first title and chapter
 		
@@ -160,8 +208,161 @@ class main_window:
 		self.set_buttons()
 		
 
+	def get_surface(self,element):
+		
+		""" Return data from a video, to be able to compute the optimal bitrate to fullfill the disk.
+			SIZE is the size in kbytes
+			SURFACE is the surface in square pixels
+			LENGTH is the efective length in seconds
+			AUDIORATE is the audio rate """
+		
+		length = float(element["olength"])
+		if element["cutting"]!=0: # we want only half the file
+			length/=2
+
+		surface = float(element["width"]*element["height"])
+		
+		surface/=84480.0 # normalize surface respect to 352x240
+		
+		if (element["copy_audio"]):
+			audiorate=element["oarate"]
+		else:
+			audiorate=element["arate"]
+
+		if (element["ismpeg"]):
+			size=element["filesize"]/1000
+		else:
+			size=(length*(element["vrate"]+audiorate))/8
+
+		return size,surface,length,audiorate
+
+
+	def get_number(self,line):
+		
+		pos=line.find(":")
+		if pos==-1:
+			return -1
+		
+		return int(line[pos+1:])
+
+
+	def get_cores(self):
+		
+		""" Returns the number of cores available in the system """
+		
+		try:
+			proc=open("/proc/cpuinfo","r")
+		except:
+			return 1 # if we can't open /PROC/CPUINFO, return only one CPU (just in case)
+		
+		cpu_list=[]
+		physical_id=-1
+		core_id=-1
+		while(True):
+			line=proc.readline()
+			if line=="":
+				break
+			if line[:9]=="processor":
+				current_id=-1
+				current_cores=-1
+				continue
+
+			if (line[:11]=="physical id"):
+				physical_id=self.get_number(line)
+			
+			if (line[:7]=="core id"):
+				core_id=self.get_number(line)
+
+			if (physical_id!=-1) and (core_id!=-1):
+				not_found=True
+				for element in cpu_list:
+					if (element[0]==physical_id) and (element[1]==core_id):
+						not_found=False
+						break
+				if not_found:
+					cpu_list.append([physical_id,core_id])
+				current_id=-1
+				current_cores=-1
+		
+		ncores=len(cpu_list)
+		if(ncores==0):
+			return 1
+		else:
+			return ncores
+
 	# Callbacks
 
+	def on_autosize_clicked(self,widget):
+		
+		""" Adjust the videorate of each file to ensure that the disk usage is optimal """
+		
+		fixed_size=[]
+		variable_size=[]
+		
+		# First, set the non-adjustable bitrate videos in a different list
+		
+		for element in self.structure:
+			for element2 in element[1:]:
+				if element2["ismpeg"]:
+					fixed_size.append(element2)
+				else:
+					variable_size.append(element2)
+		
+		discsize,minrate,maxrate=devede_other.get_dvd_size(self.tree,self.disctocreate)	
+		discsize*=1000 # size in kbytes
+		
+		size_error=False
+		dowhile=True
+		while(dowhile):
+			dowhile=False
+			v_fixed_size=0
+			for element in fixed_size:
+				size,surface,length,arate = self.get_surface(element)
+				subrate=8*len(element["sub_list"])
+				v_fixed_size+=size+int((subrate*element["olength"])/8)
+
+			available_size=discsize-v_fixed_size
+				
+			if (available_size<0): # the fixed_size videos need more disk space than the currently available
+				size_error=True
+				break
+			
+			if (len(variable_size)==0):
+				break
+			
+			total_len=0.0
+			total_surface=0.0
+			for element in variable_size:
+				size,surface,length,arate = self.get_surface(element)
+				total_len+=length
+				total_surface+=surface*length
+				
+			for element in variable_size:
+				subrate=8*len(element["sub_list"])
+				size,surface,length,arate = self.get_surface(element)
+				videorate=8*(float(available_size))*((surface*length)/total_surface)
+				videorate/=length
+				videorate-=(arate+subrate)
+				element["vrate"]=int(videorate)
+				print int(videorate)
+				if videorate<minrate:
+					element["vrate"]=minrate
+					dowhile=True
+					fixed_size.append(element)
+					variable_size.remove(element)
+					break
+				if videorate>maxrate:
+					element["vrate"]=maxrate
+					dowhile=True
+					fixed_size.append(element)
+					variable_size.remove(element)
+					break
+
+		if size_error:
+			devede_dialogs.show_error(self.gladefile,_("Too many videos for this disk size.\nPlease, select a bigger disk type or remove some videos."))
+		
+		self.set_video_values()
+
 
 	def wmain_delete_event_cb(self,widget,signal):
 	
@@ -182,7 +383,7 @@ class main_window:
 		if retval==-5:
 			gtk.main_quit()
 		return
-
+	
 
 	def draganddrop(self,widget,drag_context, x, y, selection, info, time):
 	
@@ -191,6 +392,14 @@ class main_window:
 		converter=devede_newfiles.newfile(self.global_vars["PAL"],self.disctocreate)
 		
 		list=converter.split_dnd(selection.data)
+		
+		if (len(list)==1): # check if it's a configuration file
+			filename=list[0].lower()
+			if len(filename)>6:
+				if filename[-7:]==".devede":
+					self.on_devede_open_activate(None, list[0])
+					return
+		
 		fine=True
 		list2=[]
 		for element in list:
@@ -231,6 +440,15 @@ class main_window:
 	
 	def on_main_go_clicked(self,widget):
 	
+		if self.disctocreate=="dvd":
+			total=0
+			for title in self.structure:
+				total+=len(title)-1
+			
+			if total>99:
+				devede_dialogs.show_error(self.gladefile,(_("Your project contains %(X)d movie files, but the maximum is 99. Please, remove some files and try again.")) % {"X":total})
+				return
+	
 		actions=0	
 		w=self.tree.get_widget("only_convert")
 		if w.get_active():
@@ -245,12 +463,22 @@ class main_window:
 		self.global_vars["number_actions"]=actions
 		w=self.tree.get_widget("erase_files")
 		self.global_vars["erase_temporary_files"]=w.get_active()
-	
+		
+		print "Threads: "+str(self.global_vars["multicore"])
 		conversor=devede_convert.create_all(self.gladefile,self.structure,self.global_vars,self.callback2)
 		if conversor.create_disc():
 			self.window.hide()
 	
 	
+	def on_multicore_toggled(self,widget):
+		
+		w=self.tree.get_widget("multicore")
+		if w.get_active():
+			self.global_vars["multicore"]=self.get_cores()
+		else:
+			self.global_vars["multicore"]=1	
+	
+	
 	# titles-related callbacks
 	
 	
@@ -369,6 +597,24 @@ class main_window:
 		self.set_buttons()
 
 
+	def on_main_preview_clicked(self,widget):
+	
+		title,chapter=self.get_marked()
+		self.global_vars["erase_temporary_files"]=True
+		self.global_vars["number_actions"]=1
+		tmp_structure=[["",self.structure[title][chapter]]]
+		converter=devede_convert.create_all(self.gladefile,tmp_structure,self.global_vars,self.callback3)
+		self.global_vars["temp_folder"]=converter.preview(self.global_vars["temp_folder"])
+		print "Temp folder changed to "+str(self.global_vars["temp_folder"])
+
+
+	def callback3(self):
+		
+		""" This method is called after a preview """
+		
+		return None # do nothing
+
+
 	def on_filesup_clicked(self,widget):
 
 		""" Moves a chapter up in the list """
@@ -433,7 +679,8 @@ class main_window:
 	def on_devede_new_activate(self,widget):
 	
 		""" Callback for NEW menu item,	where it asks for confirmation """
-	
+		
+		print self.structure
 		w = devede_dialogs.ask_erase_all(self.gladefile)
 		retval=w.run()
 		w = None
@@ -441,12 +688,12 @@ class main_window:
 			(self.callback)()
 
 	
-	def on_devede_open_activate(self,widget):
+	def on_devede_open_activate(self,widget,filename=None):
 		
 		""" Callback for OPEN menu item """
 		
-		load = devede_loadsave.load_save_config(self.gladefile,self.structure,self.global_vars)
-		load.load()
+		load = devede_loadsave.load_save_config(self.gladefile,self.structure,self.global_vars,self.tree)
+		load.load(filename)
 		if load.done==False:
 			load=None
 			return
@@ -487,7 +734,7 @@ class main_window:
 		
 		""" Saves the current structure """
 		
-		save=devede_loadsave.load_save_config(self.gladefile,self.structure,self.global_vars)
+		save=devede_loadsave.load_save_config(self.gladefile,self.structure,self.global_vars,self.tree)
 		save.save(mode)
 		self.set_title()
 		save=None
@@ -577,6 +824,9 @@ class main_window:
 		del_chapter=self.tree.get_widget("del_chapter")
 		del_chapter.set_sensitive(chapter_marked)
 		
+		main_preview=self.tree.get_widget("main_preview")
+		main_preview.set_sensitive(chapter_marked)
+		
 		prop_chapter=self.tree.get_widget("prop_chapter")
 		prop_chapter.set_sensitive(chapter_marked)
 		
@@ -620,8 +870,9 @@ class main_window:
 			w.set_text(str(found["arate"]))
 			
 			w=self.tree.get_widget("eleng")
-			length=int(((found["vrate"]+found["arate"])*found["olength"])/8000)
-			w.set_text(str(length))
+			length=devede_other.calcula_tamano_parcial(found["vrate"],found["arate"],found["filesize"],found["olength"],len(found["sub_list"]),found["ismpeg"],found["cutting"])
+			#length=int(((found["vrate"]+found["arate"])*found["olength"])/8000)
+			w.set_text(str(int(length/1000)))
 			
 			w=self.tree.get_widget("achap")
 			if found["lchapters"]==0:
@@ -667,26 +918,8 @@ class main_window:
 		total=devede_other.calcula_tamano_total(self.structure)
 		total/=1000
 		
-		w=self.tree.get_widget("dvdsize")
-		active=w.get_active()
+		tamano,minvrate,maxvrate = devede_other.get_dvd_size(self.tree,self.disctocreate)
 		
-		# here we choose the size in Mbytes for the media
-		if 0==active:
-			tamano=180.0
-		elif 1==active:
-			tamano=710.0
-		elif 2==active:
-			tamano=790.0
-		elif 3==active:
-			tamano=1200.0
-		elif 4==active:
-			tamano=4500.0
-		else:
-			tamano=8000.0
-		
-		tamano*=0.92 # a safe margin of 8% to ensure that it never will be bigger
-					 # (it's important to have in mind the space needed by disk structures like
-					 # directories, file entries, and so on)
 		w=self.tree.get_widget("usage")
 		if total>tamano:
 			w.set_fraction(1.0)
@@ -771,9 +1004,8 @@ class main_window:
 		
 	def create_new_structure(self,number):
 	
-		nombre=_("Title")
-		nombre+=" "+str(number)
+		name=_("Title %(X)d") % {"X":int(number)}
 		var={}
-		var["nombre"]=nombre
+		var["nombre"]=name
 		var["jumpto"]="menu" # can be MENU, FIRST, NEXT, LAST or LOOP
 		return [var]
