@@ -26,13 +26,13 @@ import sys
 import re
 import shutil
 
-from devede_other import *
+import devede_other
 
 import devede_executor
 
 class video_converter(devede_executor.executor):
 	
-	def __init__(self,videofile,filename,filefolder,progresbar,proglabel,disctype,title,chapter,seconds=0):
+	def __init__(self,videofile,filename,filefolder,progresbar,proglabel,disctype,title,chapter,threads,seconds):
 
 		""" This class converts a video file to MPEG-1 or MPEG-2 format
 
@@ -47,7 +47,7 @@ class video_converter(devede_executor.executor):
 		
 		devede_executor.executor.__init__(self,filename,filefolder,progresbar)
 		self.printout=False
-		
+
 		self.percent2=120
 		if seconds==0:
 			self.divide=float(videofile["olength"])
@@ -79,7 +79,7 @@ class video_converter(devede_executor.executor):
 				# links do not work on windows, so just copy the file
 				self.launch_shell('copy "'+videofile["path"] +'" "'+currentfile+'"',output=False)
 			else:
-				if videofile["subtitles"]=="":
+				if len(videofile["sub_list"])==0:
 					self.launch_shell('ln -s "'+videofile["path"]+'" "'+currentfile+'"',output=False)
 				else:
 					self.launch_shell('cp "'+videofile["path"]+'" "'+currentfile+'"',output=False)
@@ -109,6 +109,7 @@ class video_converter(devede_executor.executor):
 		copy_audio=videofile["copy_audio"]
 		sound51=videofile["sound51"]
 		gop12=videofile["gop12"]
+		audiostream=videofile["audio_stream"]
 	
 		if aspect_ratio_original<1.3:
 			aspect_ratio_original=float(videofile["owidth"])/(float(videofile["oheight"]))
@@ -117,32 +118,29 @@ class video_converter(devede_executor.executor):
 	
 		max_videorate=int(videorate*1.5)
 		min_videorate=int(videorate*0.75)
-		if disctype=="vcd":
-			max_videorate=1152
-			min_videorate=1152
-		elif (disctype=="svcd") or (disctype=="cvd"):
-			if (max_videorate+audiorate)>2550:
-				max_videorate=2550-audiorate
-			if min_videorate<600:
-				min_videorate=600
-		elif (disctype=="dvd") or (disctype=="divx"):
-			if (max_videorate)>8000:
-				max_videorate=8000
-			if min_videorate<400:
-				min_videorate=400
-		else:
-			print ("Error, disc format is not DVD, DIVX, VCD, SVCD or CVD. Contact with the author")
-			print disctype
-			sys.exit(1)
-	
+		
+		dsize,minvid,maxvid=devede_other.get_dvd_size(None,disctype)
+		
+		if max_videorate>maxvid:
+			max_videorate=maxvid
+		if min_videorate<minvid:
+			min_videorate=minvid
+			
 		if videofile["blackbars"]==0: # check if has to add black bars
 			addbars=True
-			if (resx_original%2)==1:
-				resx_original+=1
-			if (resy_original%2)==1:
-				resy_original+=1
-			resx_inter=resx_original
-			resy_inter=int((resy_original*aspect_ratio_original)/aspect_ratio_final)
+			if (videofile["rotate"]==90) or (videofile["rotate"]==270):
+				resx_original2=resy_original
+				resy_original2=resx_original
+			else:
+				resx_original2=resx_original
+				resy_original2=resy_original
+
+			if (resx_original2%2)==1:
+				resx_original2+=1
+			if (resy_original2%2)==1:
+				resy_original2+=1
+			resx_inter=resx_original2
+			resy_inter=int((resy_original2*aspect_ratio_original)/aspect_ratio_final)
 			if (resy_inter%2)==1:
 				resy_inter+=1
 			
@@ -151,8 +149,8 @@ class video_converter(devede_executor.executor):
 			# too much work for so little profit
 			
 			if ((resy_inter<resy_original) or (resy_original+5>resy_inter)):
-				addbars=False 
-	
+				addbars=False
+
 		if addbars==False:
 			resx_inter=resx_original
 			resy_inter=resy_original
@@ -188,6 +186,26 @@ class video_converter(devede_executor.executor):
 			else:
 				command_var.append("lavc")
 		
+		if (audiostream!=10000):
+			command_var.append("-aid")
+			command_var.append(str(audiostream))
+		
+		telecine=False
+		if final_framerate==30:
+			if (framerate==24) and ((disctype=="dvd") or (disctype=="divx")):
+				str_final_framerate="24000/1001"
+				keyintv=15
+				telecine=True
+			else:
+				str_final_framerate="30000/1001"
+				keyintv=18
+		else:
+			str_final_framerate=str(int(final_framerate))
+			keyintv=15
+		
+		if gop12:
+			keyintv=12
+		
 		command_var.append("-ovc")
 		command_var.append("lavc")
 		if (disctype!="divx"):
@@ -195,7 +213,10 @@ class video_converter(devede_executor.executor):
 			command_var.append("mpeg")
 			command_var.append("-mpegopts")
 			if disctype=="dvd":
-				command_var.append("format=dvd:tsaf")
+				if telecine:
+					command_var.append("format=dvd:tsaf:telecine")
+				else:
+					command_var.append("format=dvd:tsaf")
 			elif disctype=="vcd":
 				command_var.append("format=xvcd")
 			elif (disctype=="svcd") or (disctype=="cvd"):
@@ -218,20 +239,6 @@ class video_converter(devede_executor.executor):
 		if (audiodelay!=0.0) and (copy_audio==False):
 			command_var.append("-delay")
 			command_var.append(str(audiodelay))
-
-		if final_framerate==30:
-			if (framerate==24) and ((disctype=="dvd") or (disctype=="divx")):
-				str_final_framerate="24000/1001"
-				keyintv=15
-			else:
-				str_final_framerate="30000/1001"
-				keyintv=18
-		else:
-			str_final_framerate=str(int(final_framerate))
-			keyintv=15
-		
-		if gop12:
-			keyintv=12
 			
 		if sound51:
 			command_var.append("-channels")
@@ -245,19 +252,72 @@ class video_converter(devede_executor.executor):
 			command_var.append("DX50")
 
 		lineatemp=""
-	
 		acoma=False;
-	
+		
+		extra_params=videofile["params_vf"] # take the VF extra params
+		while (extra_params!=""):
+			extra_params,new_param=devede_other.get_new_param(extra_params)
+			if (new_param!="") and (new_param!=','):
+				while (len(new_param)>1) and (new_param[0]==','):
+					new_param=new_param[1:]
+				while (len(new_param)>1) and (new_param[-1]==','):
+					new_param=new_param[:-1]
+				if acoma:
+					lineatemp+=","
+				lineatemp+=new_param
+				acoma=True
+		
+		vmirror=0
+		hmirror=0
+		
 		if videofile["deinterlace"]!="none":
+			if acoma:
+				lineatemp+=","
 			lineatemp+="pp="+videofile["deinterlace"]
 			acoma=True
-
+			
+		if videofile["rotate"]==180:
+			vmirror=1-vmirror
+			hmirror=1-hmirror
+		
+		if videofile["vmirror"]:
+			vmirror=1-vmirror
+		
+		if videofile["hmirror"]:
+			hmirror=1-hmirror
+		
+		if vmirror==1:
+			if acoma:
+				lineatemp+=","
+			lineatemp+="flip"
+			acoma=True
+		
+		if hmirror==1:
+			if acoma:
+				lineatemp+=","
+			lineatemp+="mirror"
+			acoma=True
+		
+		print "Addbars "+str(addbars)+" resx_o "+str(resx_original)+" resy_o "+str(resy_original)
+		print "resx_i "+str(resx_inter)+" resy_i "+str(resy_inter)
 		if addbars and ((resx_inter!=resx_original) or (resy_inter!=resy_original)):
 			if acoma:
 				lineatemp+=","
 			lineatemp+="expand="+str(resx_inter)+":"+str(resy_inter)+":"+str(addx)+":"+str(addy)
 			acoma=True
 
+		if videofile["rotate"]==90:
+			if acoma:
+				lineatemp+=","
+			lineatemp+="rotate=1"
+			acoma=True
+		
+		if videofile["rotate"]==270:
+			if acoma:
+				lineatemp+=","
+			lineatemp+="rotate=2"
+			acoma=True
+
 		if (resx_inter!=resx_final) or (resy_inter!=resy_final):
 			if acoma:
 				lineatemp+=","
@@ -275,8 +335,11 @@ class video_converter(devede_executor.executor):
 			command_var.append(lineatemp)
 
 		command_var.append("-lavcopts")
-	
-		lavcopts="vcodec="
+		
+		lavcopts=""
+		if threads>1:
+			lavcopts="threads="+str(threads)+":"
+		lavcopts+="vcodec="
 		if disctype=="vcd":
 			lavcopts+="mpeg1video"
 		elif disctype=="divx":
@@ -323,12 +386,29 @@ class video_converter(devede_executor.executor):
 		else:
 			lavcopts+=":aspect=4/3"
 
+		extra_params=videofile["params_lavc"] # take the LAVC extra params
+		while (extra_params!=""):
+			extra_params,new_param=devede_other.get_new_param(extra_params)
+			if (new_param!="") and (new_param!=':'):
+				while (len(new_param)>1) and (new_param[0]==':'):
+					new_param=new_param[1:]
+				while (len(new_param)>1) and (new_param[-1]==':'):
+					new_param=new_param[:-1]
+				lavcopts+=":"+new_param
 		command_var.append(lavcopts)
-
 	
 		if (disctype=="divx") and (copy_audio==False):
 			lameopts="abr:br="+str(audiorate)
 			command_var.append("-lameopts")
+			extra_params=videofile["params_lame"] # take the LAME extra params
+			while (extra_params!=""):
+				extra_params,new_param=devede_other.get_new_param(extra_params)
+				if (new_param!="") and (new_param!=':'):
+					while (len(new_param)>1) and (new_param[0]==':'):
+						new_param=new_param[1:]
+					while (len(new_param)>1) and (new_param[-1]==':'):
+						new_param=new_param[:-1]
+					lameopts+=":"+new_param
 			command_var.append(lameopts)
 	
 		currentfile=self.create_filename(filefolder+filename,title,chapter,disctype=="divx")
@@ -339,11 +419,13 @@ class video_converter(devede_executor.executor):
 	
 		extra_params=videofile["params"] # take the extra params
 		while (extra_params!=""):
-			extra_params,new_param=get_new_param(extra_params)
+			extra_params,new_param=devede_other.get_new_param(extra_params)
 			if new_param!="":
 				command_var.append(new_param)
 
 		self.print_error=_("Conversion failed.\nIt seems a bug of Mencoder.")
+		if (videofile["params"]!="") or (videofile["params_vf"]!="") or (videofile["params_lavc"]!="") or (videofile["params_lame"]!=""):
+			self.print_error+="\n"+_("Also check the extra params passed to Mencoder for syntax errors.")
 		self.error_not_done=True
 		self.launch_program(command_var,read_chars=300)
 
